require 'uu/os/vuc/action_helper'

module UU
  module OS
    module VUC

      # Class is used for building action response which cause user navigation, opens dialog or trigger file download.
      # Action can be:
      #   * back - Returns to previous Use case, actual use case is finished
      #   * super uc - Execute super Use case instead of this one and this one is finished
      #   * forward - Navigate to another Use case, actual use case is finished
      #   * dialog - Open modal dialog, actual use case is not finished but only suspended
      #   * progress dialog - Open progress dialog, , actual use case is not finished but only suspended
      #   * download - Creates action for launching download from client (browser)
      # @example
      #   def on_submit(ctx)
      #     uu_uri = ctx.uu_uri
      #
      #     # ...
      #
      #     result = {format: 'PDF'}
      #
      #     if #...
      #
      #       info_msg = UU::OS::GVC::Message.new({})
      #       return ctx.action.forward(uu_uri, :uuMessages => [info_msg])
      #
      #     else
      #
      #       return ctx.action.back(:uuResult => result)
      #
      #     end
      #   end
      class Action

        # @private
        ACTION_JSON_SCHEMA_URI = "ues:SYSTEM:UU.OS.VUC/ACTION_MESSAGE-SCHEMA-V3" # :nodoc:

        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        def initialize(ctx)
          @ctx = ctx
        end

        # Cause back action, user is navigated to previous use case. Actual use case is finished.
        # @example
        #   # back action
        #   return ctx.action.back()
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at previous use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #   {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #   })
        #   return ctx.action.back(:uuMessages => [info_msg])
        #
        #   # include result to back action
        #   return ctx.action.back(:uuResult => 'Some result')
        #
        #   # combination of result and messages
        #   return ctx.action.back(:uuMessages => [info_msg], :uuResult => 'Some result')
        #
        # @param [Hash] params Additional parameters.
        # @option params [Array] :uuMessages Array of {UU::OS::GVC::Message} objects used as message to user.
        # @option params [Object] :uuResult Result can be small arbitrary object serializable to JSON.
        # @return [UU::OS::VUC::Action] Returns this instance.
        def back(params = {})
          @action = UU::OS::VUC::ActionHelper.create_back_uri(@ctx, params)
          return self
        end

        # Cause super use case, user is navigated to the super use case implementation. Actual use case is finished.
        # @example
        #   # cause super uc action
        #   return ctx.action.execute_super_uc()
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at super use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #   {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #   })
        #   return ctx.action.execute_super_uc(:uuMessages => [info_msg])
        #
        # @param [Hash] params Additional parameters.
        # @option params [Array] :uuMessages
        # @return [UU::OS::VUC::Action] Returns this instance.
        def execute_super_uc(params = {})
          @action = UU::OS::VUC::ActionHelper.create_super_uc_uri(@ctx, params)
          return self
        end

        # Navigate to use case passed by 'uu_uri' parameter. Actual use case is finished.
        #
        # @example
        #   #navigate to artifact's default action
        #   return ctx.action.forward('ues:TER:ART')
        #
        #   # navigate to UC1
        #   return ctx.action.forward('ues:TER:ART?UC1')
        #
        #   # navigate to UC1 with some params
        #   return ctx.action.forward('ues:TER:ART?UC1&name=Thor&age=41')
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at next use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #    {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #     })
        #   return ctx.action.forward('ues:TER:ART?UC1&name=Thor&age=41', :uuMessages => [info_msg])
        #
        #   # same as previous, including result
        #   return ctx.action.forward('ues:TER:ART?UC1&name=Thor&age=41', :uuResult => 'Some result')
        #
        #   # combination of result and messages
        #   return ctx.action.forward('ues:TER:ART?UC1&name=Thor&age=41', :uuMessages => [info_msg], :uuResult => 'Some result')
        #
        # @param [UU::OS::UESURI, String] uu_uri Uri of Use case
        # @param [Hash] params Adsditional parameters.
        # @option params [Array] :uuMessages
        # @return [UU::OS::VUC::Action] Returns this instance.
        def forward(uu_uri, params = {})
          @action = UU::OS::VUC::ActionHelper.create_forward_uri(@ctx, uu_uri, params)
          return self
        end

        # Action opens use case in modal window. Actual use case is not finished but only suspended.
        #
        # @example
        #   # opens use case in modal window
        #   return ctx.action.open_dialog('ues:TER:DIALOG?VUC&name=Thor')
        #
        #   # opens dialog with specified width and height
        #   return ctx.action.open_dialog('ues:TER:DIALOG?VUC', :width => 400, :height => 300)
        #
        #   # opens dialog with callback action dialogClosed, it means that on_dialog_closed method will be called when
        #   # dialog is closed
        #   return ctx.action.open_dialog('ues:TER:DIALOG?VUC', :width => 400, :height => 300, :uuCallback => 'dialogClosed')
        #
        # @param [UU::OS::UESURI, String] uu_uri The use case to show in the dialog.
        # @param [Hash] params Additional parameters.
        # @option params [Integer] :width Dialog width
        # @option params [Integer] :height Dialog height
        # @option params [String] :uuCallback Callback action.
        # @return [UU::OS::VUC::Action] Returns this instance.
        def open_dialog(uu_uri, params = {})
          @action = UU::OS::VUC::ActionHelper.create_dialog_uri(@ctx, uu_uri, params)
          return self
        end

        # Action opens progress dialog that waits for specific process. Actual use case is not finished but only suspended.
        #
        # @example
        #   # get process uri
        #   process_uri = #...
        #
        #   # open progress dialog, that waits for specific process
        #   return ctx.action.open_progress_dialog(process_uri)
        #
        #   # open progress dialog, that waits for specific resultCode
        #   return ctx.action.open_progress_dialog(process_uri, :resultCode => 'ARTIFACT_CREATED')
        #
        #   # open progress dialog with callback action progressDialogClosed, it means that on_progress_dialog_closed method will be called
        #   # when dialog is closed
        #   return ctx.action.open_progress_dialog(process_uri, :resultCode => 'ARTIFACT_CREATED', :uuCallback => 'progressDialogClosed')
        #
        # @param [UU::OS::UESURI, String] process_uri UESURI of asynchronous process.
        # @param [Hash] params Additional parameters.
        # @option params [Integer] :resultCode Code of process event providing result.
        # @option params [String] :uuCallback Callback action.
        # @return [UU::OS::VUC::Action] Returns this instance.
        def open_progress_dialog(process_uri, params = {})
          @action = UU::OS::VUC::ActionHelper.create_progress_dialog_uri(@ctx, process_uri, params)
          return self
        end

        # Creates action for launching download from client (browser).
        #
        # @example
        #   #download attachment
        #   return ctx.action.download('ues:TER:ART:ATTACHMENT')
        #
        #   # download attachment with callback action downloadCallback, it means that on_download_callback method will be called
        #   return ctx.action.download('ues:TER:ART:ATTACHMENT', :uuCallbaqck => 'downloadCallback')
        #
        # @param [UU::OS::UESURI, String] uu_uri The UESURI which will end with download when used.
        # @return [UU::OS::VUC::Action] Returns this instance.
        def download(uu_uri, params = {})
          @action = UU::OS::VUC::ActionHelper.create_download_uri(@ctx, uu_uri, params)
          return self
        end

        # Serialize object to JSON in format corresponding to schema ues:SYSTEM:UU.OS.VUC/ACTION_MESSAGE-SCHEMA-V3
        #
        # @param [Hash] opt Optional parameters.
        # @see #JSON
        def to_json(opt = {})
          response = {}
          response[:schemaUri] = ACTION_JSON_SCHEMA_URI
          response[:action] = @action
          return response.to_json(opt)
        end

      end
    end
  end
end
