require 'uu/os/uesuri_builder'
require 'uu/os/uesuri'

module UU
  module OS
    module VUC

      # Class is used for building action uri which cause user navigation, opens dialog or trigger file download.
      # Action can be:
      #   * back - Returns to previous Use case, actual use case is finished
      #   * super uc - Execute super Use case instead of this one and this one is finished
      #   * forward - Navigate to another Use case, actual use case is finished
      #   * dialog - Open modal dialog, actual use case is not finished but only suspended
      #   * progress dialog - Open progress dialog, , actual use case is not finished but only suspended
      #   * download - Creates action for launching download from client (browser)
      # @example
      #   def on_init(ctx)
      #
      #     view = ctx.view
      #     button = view.component(:BUTTON)
      #
      #     button.action_uri = UU::OS::VUC::ActionHelper.create_dialog_uri(
      #         ctx,
      #         'ues:TER:DIALOG?VUC',
      #         :width => 400,
      #         :uuCallback => 'dialogClosed')
      #     return view
      #   end
      module ActionHelper

        # @private
        TARGET_TYPE_DIALOG = :DIALOG_WINDOW # :nodoc:

        # @private
        TARGET_TYPE_PROGRESS_DIALOG = :PROGRESS_DIALOG_WINDOW # :nodoc:

        # @private
        TARGET_TYPE_DOWNLOAD = :DOWNLOAD # :nodoc:

        # @private
        MESSAGES_PARAM_NAME = :uuMessages # :nodoc:

        # @private
        RESULT_PARAM_NAME = :uuResult # :nodoc:

        # @private
        BACK_ACTION_NAME = :BACK # :nodoc:

        # @private
        SUPER_UC_ACTION_NAME = :SUPER_UC # :nodoc:


        # Returns back action uri, user is navigated to previous use case. Actual use case is finished.
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   # back action
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_back_uri(ctx)
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at previous use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #   {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #   })
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_back_uri(ctx, :uuMessages => [info_msg])
        #
        #   # include result to back action
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_back_uri(ctx, :uuResult => 'Some result')
        #
        #   # combination of result and messages
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_back_uri(ctx, :uuMessages => [info_msg], :uuResult => 'Some result')
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [Hash] params Additional parameters.
        # @option params [Array] :uuMessages Array of {UU::OS::GVC::Message} objects used as message to user.
        # @option params [Object] :uuResult Result can be small arbitrary object serializable to JSON.
        # @return [UU::OS::UESURI] Action uri.
        def self.create_back_uri(ctx, params = {})
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(ctx.uu_uri)

          add_messages(uri_builder, params)
          add_result(uri_builder, params)
          uri_builder.set_action(BACK_ACTION_NAME)

          return uri_builder.to_uesuri
        end


        # Returns super use case uri, user is navigated to the super use case implementation. Actual use case is finished.
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   # cause super uc action
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_super_uc_uri(ctx)
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at super use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #   {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #   })
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_super_uc_uri(ctx, :uuMessages => [info_msg])
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [Hash] params Additional parameters.
        # @option params [Array] :uuMessages
        # @return [UU::OS::UESURI] Action uri.
        def self.create_super_uc_uri(ctx, params = {})
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(ctx.uu_uri)

          add_messages(uri_builder, params)
          uri_builder.set_action(SUPER_UC_ACTION_NAME)

          return uri_builder.to_uesuri
        end

        # Returns navigation uri to use case passed by 'uu_uri' parameter. Actual use case is finished.
        #
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   #navigate to artifact's default action
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART')
        #
        #   # navigate to UC1
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART?UC1')
        #
        #   # navigate to UC1 with some params
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART?UC1&name=Thor&age=41')
        #
        #   # same as previous, but info message is included, in this example 'Info message.' text is displayed
        #   # at next use case.
        #   info_msg = UU::OS::GVC::Message.new(
        #    {
        #        code: "CODE",
        #        severity: UU::OS::GVC::Message::MessageSeverity::INFO,
        #        text: "Info message."
        #     })
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART?UC1&name=Thor&age=41', :uuMessages => [info_msg])
        #
        #   # same as previous, including result
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART?UC1&name=Thor&age=41', :uuResult => 'Some result')
        #
        #   # combination of result and messages
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_forward_uri(ctx, 'ues:TER:ART?UC1&name=Thor&age=41', :uuMessages => [info_msg], :uuResult => 'Some result')
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [UU::OS::UESURI, String] uu_uri Uri of Use case
        # @param [Hash] params Adsditional parameters.
        # @option params [Array] :uuMessages
        # @return [UU::OS::UESURI] Action uri.
        def self.create_forward_uri(ctx, uu_uri, params = {})
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(uu_uri)

          add_messages(uri_builder, params)
          add_result(uri_builder, params)

          return uri_builder.to_uesuri
        end

        # Returns uri which opens use case in modal window. Actual use case is not finished but only suspended.
        #
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   # opens use case in modal window
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_dialog_uri(ctx, 'ues:TER:DIALOG?VUC&name=Thor')
        #
        #   # opens dialog with specified width and height
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_dialog_uri(ctx, 'ues:TER:DIALOG?VUC', :width => 400, :height => 300)
        #
        #   # opens dialog with callback action dialogClosed, it means that on_dialog_closed method will be called when
        #   # dialog is closed
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_dialog_uri(ctx, 'ues:TER:DIALOG?VUC', :width => 400, :height => 300, :uuCallback => 'dialogClosed')
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [UU::OS::UESURI, String] uu_uri The use case to show in the dialog.
        # @param [Hash] params Additional parameters.
        # @option params [Integer] :width Dialog width
        # @option params [Integer] :height Dialog height
        # @option params [String] :uuCallback Callback action.
        # @return [UU::OS::UESURI] Action uri.
        def self.create_dialog_uri(ctx, uu_uri, params = {})
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(uu_uri)
          add_target(uri_builder, TARGET_TYPE_DIALOG, params)
          add_callback(uri_builder, params)
          return uri_builder.to_uesuri
        end

        # Returns uri that opens progress dialog that waits for specific process. Actual use case is not finished but only suspended.
        #
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   # get process uri
        #   process_uri = #...
        #
        #   # open progress dialog, that waits for specific process
        #   UU::OS::VUC::ActionHelper.create_progress_dialog_uri(ctx, process_uri)
        #
        #   # open progress dialog, that waits for specific resultCode
        #   UU::OS::VUC::ActionHelper.create_progress_dialog_uri(ctx, process_uri, :resultCode => 'ARTIFACT_CREATED')
        #
        #   # open progress dialog with callback action progressDialogClosed, it means that on_progress_dialog_closed method will be called
        #   # when dialog is closed
        #   UU::OS::VUC::ActionHelper.create_progress_dialog_uri(ctx, process_uri, :resultCode => 'ARTIFACT_CREATED', :uuCallback => 'progressDialogClosed')
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [UU::OS::UESURI, String] process_uri UESURI of asynchronous process.
        # @param [Hash] params Additional parameters.
        # @option params [Integer] :resultCode Code of process event providing result.
        # @option params [String] :uuCallback Callback action.
        # @return [UU::OS::UESURI] Action uri.
        def self.create_progress_dialog_uri(ctx, process_uri, params = {})
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(process_uri)
          add_target(uri_builder, TARGET_TYPE_PROGRESS_DIALOG, params)
          add_callback(uri_builder, params)
          return uri_builder.to_uesuri
        end

        # Returns uri which cause file download from client (browser).
        #
        # @example
        #
        #   # get button component
        #   view = ctx.view
        #   button = view.component(:BUTTON)
        #
        #   #download attachment
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_download_uri(ctx, 'ues:TER:ART:ATTACHMENT')
        #
        #   # download attachment with callback action downloadCallback, it means that on_download_callback method will be called
        #   button.action_uri = UU::OS::VUC::ActionHelper.create_download_uri(ctx, 'ues:TER:ART:ATTACHMENT', :uuCallbaqck => 'downloadCallback')
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @param [UU::OS::UESURI, String] uu_uri The UESURI which will end with download when used.
        # @return [UU::OS::UESURI] Action uri.
        def self.create_download_uri(ctx, uu_uri, params = {})
          if uu_uri.nil? || (uu_uri.kind_of?(String) && uu_uri.empty?)
            return uu_uri
          end
          unless (uu_uri.kind_of?(String) && (uu_uri.start_with?('ues:') || uu_uri.start_with?('uu-uds:'))) || uu_uri.kind_of?(UU::OS::UESURI)
            raise ArgumentError.new("download_uri must be UES-URI instance or URI string containing UES-URI or UDS-URI, but it was #{uu_uri}.")
          end

          uri = UU::OS::UESURI.new(uu_uri)
          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(uri)
          add_target(uri_builder, TARGET_TYPE_DOWNLOAD, params)
          add_callback(uri_builder, params)
          # TODO this fix is because UESURIBuilder do not support different scheme than 'ues' and we
          # expect e.g. 'uu-uds'
          str_uri = uri.schema + ':' + uri_builder.to_uesuri.to_s.gsub(/^ues:/, "")
          return UU::OS::UESURI.new(str_uri)
        end

        private

        # @private
        def self.add_target(uri_builder, target_type, params = {})
          target = {}
          target[:targetType] = target_type
          target[:width] = params[:width] if params[:width]
          target[:height] = params[:height] if params[:height]
          target[:resultCode] = params[:resultCode] if params[:resultCode]

          uri_builder.add_parameter(:uuTarget, target.to_json)
        end

        # @private
        def self.add_callback(uri_builder, params = {})
          return if params[:uuCallback].nil?
          callback = {}
          callback[:action] = params[:uuCallback]
          uri_builder.add_parameter(:uuCallback, callback.to_json)
        end

        # @private
        def self.add_messages(uri_builder, params = {})
         messages = params[:uuMessages]
         return if messages.nil?
         raise "uuMessages param must be Array, but was #{messages.class}" unless messages.kind_of? Array
         messages = messages.map do |msg|
           UU::OS::GVC::Message.new(msg).raw_attrs
         end
          uri_builder.add_parameter(:uuMessages, messages.to_json)
        end

        # @private
        def self.add_result(uri_builder, params = {})
          uri_builder.add_parameter(:uuResult, params[:uuResult].to_json) if params[:uuResult]
        end

      end
    end
  end
end
