require 'uu/os/application/abstract_controller'
require 'uu/os/application/app_config_loader'
require 'uu/os/util/container_service'

module UU
  module OS
    module VUC

      # Ancestor for development of custom Visual Use Case controllers v3
      class VisualUseCaseController < UU::OS::Application::AbstractController

        # Method used for business rule check and changing view template. E.g. artifact exists, user is casted to
        # role etc.
        #
        # @note No components are not available here. Components are available from {on_init} method.
        #
        # @example
        #   # uri of default VUC
        #   UCR_URI = "ues:UNI:CDS.GUESTBOOK/COMMENT/CREATE"
        #
        #   # code of sheet with extended form
        #   EXTENDED_FORM_CODE = "EXTENDED_FORM"
        #
        #   def on_before_init(ctx)
        #
        #     # get ViewModel instance from context
        #     view = ctx.view
        #
        #     # get access role
        #     session = UU::OS::Security::Session.current_session
        #     acr = session.get_access_role(ctx.uu_uri)
        #
        #     # check some business rules with access role
        #     if !acr
        #       # in case business rules are not fit, navigate back and show error message to user.
        #       error_msg = UU::OS::GVC::Message.new({
        #               code: "CODE",
        #               severity: UU::OS::GVC::Message::MessageSeverity::ERROR,
        #               text: "You shall not pass!"
        #               })
        #       return ctx.action.back(:uuMessages => [error_msg])
        #     end
        #
        #     # some logic to find out if user can see extended form
        #     use_extended_form = true
        #
        #     if use_extended_form
        #       # set new sheet with extended form
        #       new_template = UU::OS::UESURIBuilder.parse_uesuri(UCR_URI).set_object_code(EXTENDED_FORM_CODE).to_uesuri
        #       view.template_uri(new_template)
        #       return view
        #     end
        #
        #     # if you do not set template_uri on view or returns nil so default is used
        #   end
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_before_init(ctx)
        end

        # Method used for filling initial values to components. E.g. acquired from Object Store. All components are
        # available here now.
        #
        # @example
        #   def on_init(ctx)
        #     view = ctx.view
        #
        #     # set inital value to component with code 'TEXT_AREA'
        #     view.component(:TEXT_AREA)
        #     view.value = "Some init value"
        #
        #     # get all components whose code ends with 'BUTTON', e.g. 'CANCEL_BUTTON', 'ADD_BUTTON' and disable them
        #     buttons = view.find_components("BUTTON$")
        #     buttons.each do |button|
        #       button.disabled = true
        #     end
        #
        #     # you must return view, otherwise changes will not take effect
        #     return view
        #   end
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_init(ctx)
        end

        # Method used for handling component changes.
        #
        # @example
        #   def on_value_change(ctx)
        #
        #     view = ctx.view
        #
        #     # get component which trigger event
        #     input = view.event_source_component # UU::OS::Form::Input
        #w
        #     # if input size is greater than 5 enable submit button
        #     submit = view.component(:SUBMIT_BUTTON)
        #     if input.value.size > 5
        #       submit.disabled = false
        #     else
        #       submit.disabled = true
        #     end
        #     return view
        #   end
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_value_change(ctx)
        end

        # Method used for process result of this use case, usually ands up with navigation.
        #
        # @example
        #   def on_submit(ctx)
        #
        #     view = ctx.view
        #
        #     # get components value
        #     name = view.component(:NAME_INPUT)
        #     age = view.component(:AGE_INPUT)
        #
        #     # check if name is not empty
        #     if (name.nil?) || (name.empty?)
        #       error_msg = UU::OS::GVC::Message.new(
        #           {
        #               code: "CODE",
        #               severity: UU::OS::GVC::Message::MessageSeverity::ERROR,
        #               text: "Name can not be empty!"
        #           })
        #
        #       view.add_message(error_msg)
        #       return view
        #     end
        #
        #     # call some business logic
        #     UU::OS::Model.create(name, age)
        #
        #     # navigate user to main object
        #     return ctx.action.forward(ctx.uu_uri)
        #   end
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_submit(ctx)
        end

        # Method used for handling cancel actions.
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_cancel(ctx)
        end

        # Callback method for processing dialog result. If dialog set result, it is available in reserved parameter
        # 'uuResult'
        #
        # @example
        #
        #   def on_dialog_closed(ctx)
        #
        #     # get dialog result from reserved parameter :uuResult
        #     result = ctx.parameters[:uuResult]
        #
        #     if !result.nil?
        #       view = ctx.view
        #       # if dialog result is available set it as value of component
        #       format = view.component(:FORMAT_TEXT)
        #       format.value = result
        #       return view
        #     end
        #
        #   end
        #
        # @param [UU::OS::VUC::VisualUseCaseContext] ctx Visual use case context.
        # @return [Object, nil] Any object that is serializable to View or Action response message, it is
        #   typically one from:
        #   * {UU::OS::VUC::ViewModel}
        #   * {UU::OS::VUC::Action}
        def on_dialog_closed(ctx)
        end

        # Controller need not to implement all possible event methods. This method
        # provides default behavior of all unimplemented events.
        #
        # @private
        def method_missing(method, *args, &block) # :nodoc:
          # Do not touch view in case of unimplemented event with exception
          # of submit event which must be always implemented.
          if (method == :on_submit)
            raise "Controller #{self.class} does not define on_submit method."
          elsif (method.to_s[/^on_/] == nil) || (args.size != 1)
            # Invoked method was not "event" method with view parameter - throw regular method missing exception.
            raise NameError.new("undefined local variable or method '#{method.to_s}' for #{self.class}")
          end
          return nil
        end

      end
    end
  end
end